function [results, model, data] = ADMIRE_detect(outputFiles, dataFiles, ignore, preprocFlags, params, FDR, varargin)
% function [results, model] = ADMIRE_detect(outputFiles, dataFiles, ignore, preprocFlags, params, FDR)
% 
% ADMIRE detects recurrent aberrations accross multiple genomic profiles.
% 
%   INPUT:
%       dataFiles - Input file names containing aCGH profiles
%           dataFiles.samples = .tsv file name containing sample ID strings in one column.
%           dataFiles.markers = .tsv file name containing marker info in each column:
%                                            marker_name(string) \t chromosome(1-24) \t Position(integer)
%           dataFiles.logr        = .tsv file name containing S*P numeric matrix with raw log ratio
%                                            values (S = # of samples, P = # of probes).
%                                            Set to zero if not applicable
%           dataFiles.logrSeg = same as dataFiles.logr except probe values take on segmented log 
%                                           ratios. Set to zero if not applicable
%                           
%       ignore - Information about which markers to ignore in the analysis
%           ignore.chroms       = vector of chromosomes to ignore e.g. [23, 24]
%                                            DEFAULT (set to zero): Don't ignore any
%           ignore.probesFile = .tsv file name containing column of marker names to ignore
%                                           (e.g. copy number variations)
%                                           DEFAULT(set to zero): Don't ignore any
% 
%       preprocFlags - Flags providing information about
%           preprocFlags.seg -  0 = non-segmented input, log ratios should be provided
%                                                 in  dataFiles.logr
%                                           1 = segmented input, segmented log ratios should
%                                                 be provided in dataFiles.logrSeg
%                                           NO DEFAULT
%           preprocFlag.sys  -  Probes might have systematic measurement biases accross all samples. 
%                                          Part of this systematic bias might be related to genomic position which 
%                                          must be removed beforehand using a dewaving algorithm. The remaining 
%                                          systematic bias should be unrelated to genomic position and this flag 
%                                          indicates whether the model should account for this bias.   
%                                       - 0 = do not account for systematic variation (DEFAULT)
%                                       - 1 = account for systematic variation (if preprocFlags.seg = 1, then the 
%                                               non-segmented log ratios are also required in order to esitmate
%                                               the increase in variation due to systematic biases in probes).
%                                       NOTE 1: To see if systematic variation is present, first appoximate the 
%                                                      measurement noise profiles (subtract biological segments). 
%                                                      Then compare the variation in the summed profile with the sum 
%                                                      of the individual profile variations. These should be almost the same 
%                                                      if no systimatic biases are present.
%                                       NOTE 2: To see if the measurement noise is unrelated to genomic
%                                                      position, sort all markers according to genomic position,
%                                                      remove all biological segments and calculate the auto-correlation.
%                                       NOTE 3: If the systematic variation is large (and related to genomic position)
%                                                      we recomend that the user employ a dewaving algorithm.
%                                       NOTE 4: If the systematic variation is large (but unrelated to genomic location) 
%                                                      we recommend that the user set proprocFlag.sys = 1 (to prevent 
%                                                      optimistic results) and  preprocFlags.seg = 1 (to reduce measurent 
%                                                      noise, including the systematic part). 
%
%       params - User specified parameters for the ADMIRE algorithm.
%          params.kw_min   - minimum Gaussian kernel width (bp) for smoothing.
%                                        DEFAULT (set to zero): twice the average distance between probes
%                                        NOTE: If sequences are sorted and not assosiated
%                                        with bp locations, simply set kw_min to the minimum number
%                                        of probes to average (with the Gaussian kernel)
%                                        e.g. default would be two.
%          params.alpha      - Smoothing resolution parameter (used in multi-scale detection)
%                                        DEFAULT (set to zero): 20
% 
%       FDR       - Event based false descovery rate
%                        No DEFAULT
% 
%  OUTPUT:
%       outputFiles - Structure with output file names. This is specified as
%                            the first parameter for ADMIRE.
%           outputFiles.events = .txt file with all the different significant
%                                            events.
%       results     - Sturcture containing all the significant events and
%                         their respective recursive levels
% 
%       model      - Contains basic field parameters and also convolution
%                         results
% 
%       NOTE: The end user may not be concerned with 'results' or 'model'. For a
%       relevant summary of the result, rather refer to the output file specified by
%       outputFiles

    thisFileDir = fileparts(which('ADMIRE_detect.m'));
    
    if ~inputConsistant(dataFiles, preprocFlags, FDR)
        results = [];
        model = [];
        data = [];
        return;
    end
    
% Select kernel file: this is a .mat file containing kernel information
% (default is a Gaussian kernel) with fields:
% 'g' - vector of genomic locations
% 'kernel' - the kernel vector of same length as 'g'. <g, kernel> uniquely
% determines the kernel at ONE kernel width.
% 'd_dg_kernel' - the derivitive of the kernel.
    kernelFile = fullfile(thisFileDir, 'kernel', 'normal.mat');
% If alpha is set to zero, select default:
    if params.alpha == 0
        params.alpha = 20;
    end
    
    if ignore.chroms == 0
        ignore.chroms = [];
    end
    
    data = getParams([], dataFiles, preprocFlags, ignore, [], 'data');
    
% If minimum kernel width is set to zero, select default:
    chromLengths = getChromLengths(data.chrom, data.mapLoc);
    if params.kw_min == 0
        numProbes      = length(data.mapLoc);
        avgProbesDist = sum(chromLengths) / (numProbes - length(chromLengths));
        params.kw_min = 2*avgProbesDist;
    end

    % Set the maximum kernel width to 1/50 of the length of the largest:
    % chromosome
    kw_max = max(chromLengths) / 50;


    % Select the kernel widhts used in the scale space
    numKernels = floor(log2(kw_max / params.kw_min));
    kws = params.kw_min * 2.^(0:numKernels);

    % Estimate null-parameters
    
    if length(varargin) == 1
        fieldParams = varargin{1}.fieldParams;
    else
        [data, fieldParams] = getNullParams(data, preprocFlags.seg, preprocFlags.sys);
    end
    
    if preprocFlags.seg == 0
        data.buffer.nextUpdate = 'lr';
    else
        data.buffer.nextUpdate = 'lrSeg';
    end

    if length(varargin) == 1
        convInfo = varargin{1}.convInfo;
        conv = varargin{1}.conv;
    else
        [data, fieldParams, convInfo, conv] = getScaleSpace(data, kws, kernelFile, fieldParams);
    end

    
    
    [data, layerSegs, layerSigns, layerMus, layerStd] = detectModulesFDR(data, fieldParams, convInfo, conv, FDR, params.alpha);
    
    results.layerSegs = layerSegs;
    results.layerSigns = layerSigns;
    results.layerMus = layerMus;
    results.layerStd = layerStd;
%     
    model.fieldParams = fieldParams;
    model.convInfo = convInfo;
    model.conv = conv;
    

    results = outputEvents(data, results, outputFiles.events);

    printf(-1,1,'DONE!\n');
end


function flag = inputConsistant(dataFiles, preprocFlags, FDR)
flag = 1;
    if FDR < 0 || FDR > 0.5
        printf(-1,1, 'ERROR: FDR must be in range (0,0.5)\n');
        flag = 0;
    end
   
    if ischar(dataFiles.samples)
        if ~exist(dataFiles.samples, 'file')
            printf(-1,1, 'ERROR: dataFiles.samples does not exist\n');
            flag = 0;
        end
    else
        printf(-1,1, 'ERROR: dataFiles.samples does not exist\n');
        flag = 0;
    end
    
    if ischar(dataFiles.markers)
        if ~exist(dataFiles.markers, 'file')
            printf(-1,1, 'ERROR: dataFiles.markers does not exist\n');
            flag = 0;
        end
    else
        printf(-1,1, 'ERROR: dataFiles.markers does not exist\n');
        flag = 0;
    end
    
    logrExist = 1;
    logrSegExist = 1;
    if ischar(dataFiles.logr)
        if ~exist(dataFiles.logr, 'file')
            logrExist = 0;
        end
    else
        logrExist = 0;
    end
     if ischar(dataFiles.logrSeg)
        if ~exist(dataFiles.logrSeg, 'file')
            logrSegExist = 0;
        end
    else
        logrSegExist = 0;
     end
    
     if (preprocFlags.seg == 0) && (~logrExist)
         printf(-1,1, 'ERROR: preprocFlags.seg == 0 but dataFiles.logr does not exist\n');
         flag = 0;
     end
     if (preprocFlags.seg == 1) && (~logrSegExist)
         printf(-1,1, 'ERROR: preprocFlags.seg == 1 but dataFiles.logrSeg does not exist\n');
         flag = 0;
     end
     if(preprocFlags.sys == 1) && (~logrExist)
         printf(-1,1, 'ERROR: preprocFlags.sys == 1 but dataFiles.logr does not exist\n');
         flag = 0;
     end

     
    
end

function chromLengths = getChromLengths(dataChroms, mapLoc)
    chroms = unique(dataChroms);
    numChroms = length(chroms);
    chromLengths = zeros(1, numChroms);
    for chromI = 1:numChroms
        currChromLocs = mapLoc(dataChroms == chroms(chromI));
        currMinLoc = min(currChromLocs);
        currMaxLoc = max(currChromLocs);
        chromLengths(chromI) = currMaxLoc - currMinLoc;
    end
end



