function [data, fieldParams ] = getNullParams(data, segFlag, sysFlag)
% Get null process parameters
% INPUT:
%   data - data structure containing logRatios and logRatiosSeg
%   segFlag - Flag indication whether segmented data should be used
%   sysFlag - Flag indicating wether systematic variation should be
%                        accounted for. 
% OUTPUT:
%   fieldParams - structure with all the null parameters, mu, sig,
%                         auto-correlation and systematic variation
 
    printf(-1, 1, 'Estimating null parameters:\n');
    if sysFlag == 1
        printf(-1,1, '\tCalculating systematic variation ...\n');
        [data, sysVar] = getParams(data, [], [], [], [], 'sysVar');
    else
        sysVar = 0;
    end
    
%     printf(-1,1,'\tCalculating approx. null region ...\n');
    
%     nullProbes = 1:numProbes;
    
    printf(-1,1,'\tEstimating mean ...\n');
    if segFlag == 0
        data.buffer.nextUpdate = 'lr';
    else
        data.buffer.nextUpdate = 'lrSeg';
    end
    
    [data, nullProbes] = getApproxNullRegion(data, sysVar);
    lrAgrNull = data.lrAgr(:,nullProbes);
    mu = mean(lrAgrNull);
    
    printf(-1,1, '\tEstimating covariance ... ');
    printf(-1,1, '');
    
    [data, R] = getParams(data,[],[],[],1:data.P,'cov');
    
    sig = sqrt(R(1));
    R = R / R(1);
   
    fieldParams.mu = mu;
    fieldParams.sig = sig;
    fieldParams.R = R;
    fieldParams.sigSys = sqrt(sysVar);
    
    if (segFlag == 1)
        fieldParams.segFlag = 1;
    else
        fieldParams.segFlag = 0;
    end
    
end



function [data, nullProbes] = getApproxNullRegion(data, sysVar)
% This function calculates an approximate null region (region with no recurrences). This is only a start
% location and will be updated interatively later
    varRatioCuttoff = 1.2;

    if data.P < 2
        nullProbes = 1:data.P;
        return
    end

    ProbesUse = logical(ones(1,data.P));

    varRatio = Inf;
    while varRatio > varRatioCuttoff
        lrAgr = data.lrAgr;
        
        [data, varNull] = getParams(data, [], [], [], ProbesUse, 'var');
        varNull  = varNull + sysVar;
        varAgr  = var(lrAgr(ProbesUse));
        muAgr   = mean(lrAgr(ProbesUse));
        lrAgr(~ProbesUse) = muAgr;

        varRatio = varAgr / varNull;
        if varRatio > varRatioCuttoff
            [~, maxVarProbesI] = sort(abs(lrAgr - muAgr), 'descend');
            maxVarProbesI = maxVarProbesI(1:ceil(0.05*sum(abs(lrAgr) ~= 0)));

            ProbesUse(maxVarProbesI) = false;

        end
        if sum(ProbesUse) <= 2
            ProbesUse = logical(ones(1,data.P));
            break;
        end
    end

    nullProbes = find(ProbesUse);
end

