function [data, returnInfo] = getParams(data, dataFiles, preprocFlags, ignore, probeI, type)
    types = {'data', 'sysVar', 'var', 'cov', 'seg'};

    if isequal(type, types{1})
        data = loadData(dataFiles, preprocFlags, ignore);
        returnInfo = 0;
    elseif isequal(type, types{2})
        [data, returnInfo] = getSysVar(data);
    elseif isequal(type, types{3})
        [data, returnInfo] = getVar(data, probeI);
    elseif isequal(type, types{4})
        [data, returnInfo] = getCov(data, probeI);
    elseif isequal(type, types{5})
        [data, returnInfo] = getSeg(data);
    else
        returnInfo = 0;
    end
end

function data = loadData(dataFiles, preprocFlags, ignore)
    tempLrFile          = './lr.dat';
    tempLrSegFile   = './lrSeg.dat'; 

%     maxMatrixMBsize = 10.00;
    maxMatrixMBsize = 300.00;
    doubleSize = whos('maxMatrixMBsize');
    doubleSize = doubleSize.bytes;
    maxMatrixNumElem = maxMatrixMBsize * 1024^2 / doubleSize;
    
    data.buffer.dS = 0;
    data.buffer.nFolds = 0;
    data.buffer.lr_sI = 0;
    data.buffer.lrSeg_sI = 0; 
    data.buffer.nextUpdate = '';
    data.buffer.lrFile = '';
    data.buffer.lrSegFile = '';
    
    data.P = 0;
    data.S = 0;
    
    data.sampleName = [];
    data.probes = [];
    data.chrom = [];
    data.mapLoc = [];
    data.relMapLoc = [];
    data.logRatios = [];
    data.logRatiosSeg = [];
    data.lrAgr = [];
    
    data.sampleName = getColStrsFromFile(dataFiles.samples);
    
    [markerNames, markerChroms, markerPos] = getMarkerInfo(dataFiles.markers);
    data.probes = markerNames;
    data.chrom = markerChroms;
    data.mapLoc = markerPos;
    
    probesRemove = getColStrsFromFile(ignore.probesFile);
    data = rel2absMapLoc(data, 1e9, ignore.chroms, probesRemove);
    
    data.P = length(data.mapLoc);
    data.S = length(data.sampleName);
    data.buffer.dS = min(max(floor(maxMatrixNumElem / data.P), 1), data.S);
    data.buffer.nFolds = ceil(data.S / data.buffer.dS);
    
    
    
%     [~, ~, probeI] = intersect(data.probes, markerNames, 'stable');
    [~, I, J] = intersect(markerNames, data.probes);
    [~, K] = sort(J, 'ascend');
    probeI = I(K);
    
    printf(-1,1,'Rewriting text files to binary format ...\n');
    
    if (preprocFlags.seg == 0) || (preprocFlags.sys == 1)
        [dataDir, dataFileName] = fileparts(dataFiles.logr);
        data.buffer.lrFile = fullfile(dataDir, sprintf('%s.dat', dataFileName));
        if exist(data.buffer.lrFile, 'file')
            delete(data.buffer.lrFile);
        end
        tsv2dat(dataFiles.logr, data.buffer.lrFile, probeI);
        
        data.buffer.nextUpdate = 'lr';
        data = loadLr(data);
    end
    if (preprocFlags.seg == 1)
        [dataDir, dataFileName] = fileparts(dataFiles.logrSeg);
        data.buffer.lrSegFile = fullfile(dataDir, sprintf('%s.dat', dataFileName));
        if exist(data.buffer.lrSegFile, 'file')
            delete(data.buffer.lrSegFile);
        end
        tsv2dat(dataFiles.logrSeg, data.buffer.lrSegFile, probeI);
        data.buffer.nextUpdate = 'lrSeg';
        data = loadLr(data);
    end
    
    if preprocFlags.seg == 0
        data.buffer.nextUpdate = 'lr';  
    else
        data.buffer.nextUpdate = 'lrSeg';  
    end
    data.lrAgr = getAggrProfile(data);
    
    
end

function sampleName = getColStrsFromFile(sampFile)
    if ischar(sampFile)
        if exist(sampFile, 'file')
            fid = fopen(sampFile, 'r');
            sampleName = textscan(fid, '%s');
            sampleName = sampleName{1};
            fclose(fid);
        else
            sampleName = {};
        end
    else
        sampleName = {};
    end
end

function [names, chrom, pos] = getMarkerInfo(markerFile)
    fid = fopen(markerFile, 'r');
    A = textscan(fid, '%s\t%f\t%f');
    fclose(fid);
    names = A{1}';
    chrom = A{2}';
    pos     = A{3}';
end

function [data]  = rel2absMapLoc(data, spacing, chromsRemove, probesRemove)


    data.mapLoc = double(data.mapLoc);

    chroms = setdiff(unique(data.chrom), chromsRemove);

    absMapLocStart = 0;
    [~, remProbeI] = intersect(data.probes, probesRemove);
    notRemProbeI = ones(1, length(data.chrom));
    notRemProbeI(remProbeI) = 0;
    probeI = zeros(1, length(data.chrom));
    for i = chroms
        probeI = probeI | data.chrom == i;
    end
    probeI = probeI & notRemProbeI;

    numSamps = length(data.sampleName);
    numProbes = sum(probeI);
    
    data.probes = data.probes(probeI);
    data.chrom = data.chrom(probeI);
    data.mapLoc = data.mapLoc(probeI);
    
    sortChroms = zeros(1, numProbes);
    relMapLoc  = zeros(1, numProbes);
    absMapLoc  = zeros(1, numProbes);
    probeNames = cell(1, numProbes);
    
    startI = 1;
    for chromI = chroms
        currChroms  = data.chrom(data.chrom == chromI);
        currMapLocs = data.mapLoc(data.chrom == chromI);
        currProbeNames = data.probes(data.chrom == chromI);
       
        [~, sortI]  = sort(currMapLocs, 'ascend');
        currLen = length(currMapLocs);

        sortChroms(1,startI:startI+currLen-1)  = currChroms(sortI);
        relMapLoc(1,startI:startI+currLen-1)   = currMapLocs(sortI);
        absMapLoc(1,startI:startI+currLen-1)   = absMapLocStart + currMapLocs(sortI);
   
        probeNames(1,startI:startI+currLen-1) = currProbeNames(sortI);

        
        absMapLocStart = absMapLoc(1,startI+currLen-1) + spacing;
        startI = startI+currLen;

    end

    [~, uniqueMapLocI] = unique(absMapLoc);
%     dataOut = data;
    data.chrom = sortChroms(uniqueMapLocI);
    data.mapLoc    = absMapLoc(uniqueMapLocI);
    data.relMapLoc = relMapLoc(uniqueMapLocI);

    if isfield(data, 'probes')
        data.probes = probeNames(uniqueMapLocI);
    end
     
end

function data = loadLr(data)
    types = {'lr', 'lrSeg'};
    if isequal(data.buffer.nextUpdate, 'lr')
        if data.buffer.lr_sI == 0
            fp = fopen(data.buffer.lrFile, 'r');
            data.logRatios = fread(fp, [data.P, data.buffer.dS], 'float')';
            fclose(fp);    
            data.buffer.lr_sI = 1;
        else
            lrNew = mod(data.buffer.lr_sI, data.buffer.nFolds)+1;
            if lrNew ~= data.buffer.lr_sI
                data.buffer.lr_sI = lrNew;
                bufferStart = (data.buffer.lr_sI-1)*data.P*data.buffer.dS*4;
                if data.buffer.lr_sI == data.buffer.nFolds
                    curr_dS = data.S - (data.buffer.nFolds - 1)*data.buffer.dS;
                else
                    curr_dS = data.buffer.dS;
                end
                fp = fopen(data.buffer.lrFile, 'r');
                fseek(fp, bufferStart, 'bof');
                data.logRatios = fread(fp, [data.P, curr_dS], 'float')';
                fclose(fp);    
            end
        end
    else
        if data.buffer.lrSeg_sI == 0
            fp = fopen(data.buffer.lrSegFile, 'r');
            data.logRatiosSeg = fread(fp, [data.P, data.buffer.dS], 'float')';
            fclose(fp);    
            data.buffer.lrSeg_sI = 1;
        else
            lrNew = mod(data.buffer.lrSeg_sI, data.buffer.nFolds)+1;
            if lrNew ~= data.buffer.lrSeg_sI
                data.buffer.lrSeg_sI = lrNew;
                bufferStart = (data.buffer.lrSeg_sI-1)*data.P*data.buffer.dS*4;
                if data.buffer.lrSeg_sI == data.buffer.nFolds
                    curr_dS = data.S - (data.buffer.nFolds - 1)*data.buffer.dS;
                else
                    curr_dS = data.buffer.dS;
                end
                fp = fopen(data.buffer.lrSegFile, 'r');
                fseek(fp, bufferStart, 'bof');
                data.logRatiosSeg = fread(fp, [data.P, curr_dS], 'float')';
                fclose(fp);    
            end
        end
        
    end

end

function tsv2dat(fileIn, fileOut, sortI)
    fidIn = fopen(fileIn, 'r');
    fidOut = fopen(fileOut, 'w');
    sampI = 1;
    
    while (1)
        textLine = fgets(fidIn);
        if isequal(textLine, -1)
            break;
        end
        sampLr = sscanf(textLine, '%f\t');
        sampLr = sampLr(sortI);
        fwrite(fidOut, sampLr, 'float');
        sampI = sampI + 1;
    end
    fclose(fidOut);
    fclose(fidIn);
end

function aggrProfile = getAggrProfile(data)
    if isequal(data.buffer.nextUpdate, 'lr');
        aggrProfile = sum(data.logRatios, 1);
        start_sI = data.buffer.lr_sI;
        next_sI = mod(data.buffer.lr_sI, data.buffer.nFolds)+1;
        while next_sI ~= start_sI
            data = loadLr(data);
            aggrProfile = aggrProfile + sum(data.logRatios, 1);
            next_sI = mod(data.buffer.lr_sI, data.buffer.nFolds)+1;
        end
    else
        aggrProfile = sum(data.logRatiosSeg, 1);
        start_sI = data.buffer.lrSeg_sI;
        next_sI = mod(data.buffer.lrSeg_sI, data.buffer.nFolds)+1;
        while next_sI ~= start_sI
            data = loadLr(data);
            aggrProfile = aggrProfile + sum(data.logRatiosSeg, 1);
            next_sI = mod(data.buffer.lrSeg_sI, data.buffer.nFolds)+1;
        end
    end
end

function [data, systematicVar] = getSysVar(data)
    function [diffAgr, diffVarAgr] = updateDiffAgr(logRatios, diffAgr, diffVarAgr)
        [numSamps,~] = size(logRatios);
        for sampI = 1:numSamps
            currDiff = diff(logRatios(sampI, :));
            diffAgr = diffAgr + currDiff;
            diffVarAgr = diffVarAgr + var(currDiff);
        end
    end
    
    diffAgr = zeros(1, data.P-1);
    diffVarAgr = 0;
    
    [diffAgr, diffVarAgr] = updateDiffAgr(data.logRatios, diffAgr, diffVarAgr);
    
    data.buffer.nextUpdate = 'lr';
    start_sI = data.buffer.lr_sI;
    next_sI = mod(data.buffer.lr_sI, data.buffer.nFolds)+1;
    while next_sI ~= start_sI
        data = loadLr(data);
        [diffAgr, diffVarAgr] = updateDiffAgr(data.logRatios, diffAgr, diffVarAgr);
        next_sI = mod(data.buffer.lr_sI, data.buffer.nFolds)+1;
    end
    
    systematicVar = 0.5*(var(diffAgr) - diffVarAgr);   
    
end

function [data, variance] = getVar(data, probes)
    if isequal(data.buffer.nextUpdate, 'lr')
        variance = sum(var(data.logRatios(:,probes), [], 2));
        start_sI = data.buffer.lr_sI;
        next_sI = mod(data.buffer.lr_sI, data.buffer.nFolds)+1;
        while next_sI ~= start_sI
            data = loadLr(data);
            variance = variance + sum(var(data.logRatios(:,probes), [], 2));
            next_sI = mod(data.buffer.lr_sI, data.buffer.nFolds)+1;
        end
    else
        variance = sum(var(data.logRatiosSeg(:,probes), [], 2));
        start_sI = data.buffer.lrSeg_sI;
        next_sI = mod(data.buffer.lrSeg_sI, data.buffer.nFolds)+1;
        while next_sI ~= start_sI
            data = loadLr(data);
            variance = variance + sum(var(data.logRatiosSeg(:,probes), [], 2));
            next_sI = mod(data.buffer.lrSeg_sI, data.buffer.nFolds)+1;
        end
    end
    
end

function [data, R] = getCov(data, probeI)
    function [autoCovAggr] = updateAutoCov(logRatios, probes, autoCovAggr)
        [numSamps, ~] = size(logRatios);
        for sampI = 1:numSamps
            if mod(sampI, 10) == 0
                printf(1,1,'%.2f%%', 100*sampI / numSamps);
            end
            autoCovFft = fft(logRatios(sampI,probes));
            autoCovFft(1) = 0;
            autoCovAggr = autoCovAggr + ifft(autoCovFft .* conj(autoCovFft));
        end
        printf(1,1, '100%%\n');   
    end

    autoCovAggr = zeros(size(data.lrAgr(probeI)));
    
    if isequal(data.buffer.nextUpdate, 'lr')
        autoCovAggr = updateAutoCov(data.logRatios, probeI, autoCovAggr);

        start_sI = data.buffer.lr_sI;
        next_sI = mod(data.buffer.lr_sI, data.buffer.nFolds)+1;
        while next_sI ~= start_sI
            data = loadLr(data);
            autoCovAggr = updateAutoCov(data.logRatios, probeI, autoCovAggr);
            next_sI = mod(data.buffer.lr_sI, data.buffer.nFolds)+1;
        end
    else
        autoCovAggr = updateAutoCov(data.logRatiosSeg, probeI, autoCovAggr);

        start_sI = data.buffer.lrSeg_sI;
        next_sI = mod(data.buffer.lrSeg_sI, data.buffer.nFolds)+1;
        while next_sI ~= start_sI
            data = loadLr(data);
            autoCovAggr = updateAutoCov(data.logRatiosSeg, probeI, autoCovAggr);
            next_sI = mod(data.buffer.lrSeg_sI, data.buffer.nFolds)+1;
        end  
    end
    
    R = autoCovAggr / length(autoCovAggr);
     
end

function [data, segInfo] = getSeg(data)
    function [cumIAgg, segStarts, segEnds] = updateSegInfo(logRatios, cumIAgg, segStarts, segEnds)
        [numSamps, ~] = size(logRatios);
        [nRows, nCols] = size(segStarts);

        for sampI = 1:numSamps
            currSegI = cumsum([0, diff(logRatios(sampI, :)) ~= 0]) + 1;
            cumIAgg = cumIAgg + currSegI;
            [~, currStarts] = unique(currSegI, 'first');
            [~, currEnds] = unique(currSegI, 'last');
            numStarts = length(currStarts);
            if numStarts > nCols
                segStarts = cat(2, segStarts, zeros(nRows, numStarts - nCols));
                segEnds = cat(2, segEnds, zeros(nRows, numStarts - nCols));
                nCols = numStarts;
            elseif numStarts < nCols
                currStarts = cat(2, currStarts, zeros(1, nCols - numStarts));
                currEnds = cat(2, currEnds, zeros(1, nCols - numStarts));
            end
            segStarts = cat(1, segStarts, currStarts);
            segEnds = cat(1, segEnds, currEnds);
            nRows = nRows + 1;
        end
    end

    data.buffer.nextUpdate = 'lrSeg';
    
    cumIAgg = zeros(1, data.P);
    segStarts = [];
    segEnds = [];
    
    [cumIAgg, segStarts, segEnds] = updateSegInfo(data.logRatiosSeg, cumIAgg, segStarts, segEnds);
    
    start_sI = data.buffer.lrSeg_sI;
    next_sI = mod(data.buffer.lrSeg_sI, data.buffer.nFolds)+1;
    while next_sI ~= start_sI
        data = loadLr(data);
        [cumIAgg, segStarts, segEnds] = updateSegInfo(data.logRatiosSeg, cumIAgg, segStarts, segEnds);
        next_sI = mod(data.buffer.lrSeg_sI, data.buffer.nFolds)+1;
    end  


    segInfo.cumIAgg = cumIAgg;
    segInfo.segStarts = segStarts;
    segInfo.segEnds = segEnds;
end



